@@ -23,6 +23,73 @@
 #include "effects.h"
 #include "preset.h"
 #include "gtkknob.h"
+#include "images/gdigi_icon.h"
+#include "gdigi_xml.h"
+
+
+static gchar* MessageID_names[] = {
+    [REQUEST_WHO_AM_I] = "REQUEST_WHO_AM_I",
+    [RECEIVE_WHO_AM_I] = "RECEIVE_WHO_AM_I",
+
+    [REQUEST_DEVICE_CONFIGURATION] = "REQUEST_DEVICE_CONFIGURATION",
+    [RECEIVE_DEVICE_CONFIGURATION] = "RECEIVE_DEVICE_CONFIGURATION",
+
+    [REQUEST_GLOBAL_PARAMETERS] = "REQUEST_GLOBAL_PARAMETERS",
+    [RECEIVE_GLOBAL_PARAMETERS] = "RECEIVE_GLOBAL_PARAMETERS",
+
+    [REQUEST_BULK_DUMP] = "REQUEST_BULK_DUMP",
+    [RECEIVE_BULK_DUMP_START] = "RECEIVE_BULK_DUMP_START",
+    [RECEIVE_BULK_DUMP_END] = "RECEIVE_BULK_DUMP_END",
+
+    [REQUEST_PRESET_NAMES] = "REQUEST_PRESET_NAMES",
+    [RECEIVE_PRESET_NAMES] = "RECEIVE_PRESET_NAMES",
+
+    [REQUEST_PRESET_NAME] = "REQUEST_PRESET_NAME",
+    [RECEIVE_PRESET_NAME] = "RECEIVE_PRESET_NAME",
+
+    [REQUEST_PRESET] = "REQUEST_PRESET",
+    [RECEIVE_PRESET_START] = "RECEIVE_PRESET_START",
+    [RECEIVE_PRESET_END] = "RECEIVE_PRESET_END",
+    [RECEIVE_PRESET_PARAMETERS] = "RECEIVE_PRESET_PARAMETERS",
+
+    [LOAD_EDIT_BUFFER_PRESET] = "LOAD_EDIT_BUFFER_PRESET",
+
+    [MOVE_PRESET] = "MOVE_PRESET",
+
+    [REQUEST_MODIFIER_LINKABLE_LIST] = "REQUEST_MODIFIER_LINKABLE_LIST",
+    [RECEIVE_MODIFIER_LINKABLE_LIST] = "RECEIVE_MODIFIER_LINKABLE_LIST",
+
+    [REQUEST_PARAMETER_VALUE] = "REQUEST_PARAMETER_VALUE",
+    [RECEIVE_PARAMETER_VALUE] = "RECEIVE_PARAMETER_VALUE",
+
+    /* version 1 and later */
+    [REQUEST_OBJECT_NAMES] = "REQUEST_OBJECT_NAMES",
+    [RECEIVE_OBJECT_NAMES] = "RECEIVE_OBJECT_NAMES",
+    [REQUEST_OBJECT_NAME] = "REQUEST_OBJECT_NAME",
+    [RECEIVE_OBJECT_NAME] = "RECEIVE_OBJECT_NAME",
+    [REQUEST_OBJECT] = "REQUEST_OBJECT",
+    [RECEIVE_OBJECT] = "RECEIVE_OBJECT",
+    [MOVE_OBJECT] = "MOVE_OBJECT",
+    [DELETE_OBJECT] = "DELETE_OBJECT",
+    [REQUEST_TABLE] = "REQUEST_TABLE",
+    [RECEIVE_TABLE] = "RECEIVE_TABLE",
+
+    [RECEIVE_DEVICE_NOTIFICATION] = "RECEIVE_DEVICE_NOTIFICATION",
+
+    [ACK] = "ACK",
+    [NACK] = "NACK",
+};
+
+const gchar*
+get_message_name(MessageID msgid)
+{
+    if (MessageID_names[msgid]) {
+        return MessageID_names[msgid];
+    }
+
+    return "Unknown";
+
+}
 
 typedef struct {
     GObject *widget;
@@ -54,7 +121,7 @@ void show_error_message(GtkWidget *parent, gchar *message)
                                             GTK_BUTTONS_OK,
                                             "%s", message);
 
-    gtk_dialog_run(GTK_DIALOG(msg));
+    (void)gtk_dialog_run(GTK_DIALOG(msg));
     gtk_widget_destroy(msg);
 }
 
@@ -184,7 +251,7 @@ static gboolean custom_value_output_cb(GtkSpinButton *spin, EffectValues *values
         if (values->type & VALUE_TYPE_EXTRA) {
             values = values->extra;
         } else {
-            g_message("custom_value_output_cb called with out of bounds value");
+            g_warning("custom_value_output_cb called with out of bounds value");
             return FALSE;
         }
     }
@@ -306,8 +373,9 @@ static void apply_widget_setting(WidgetTreeElem *el, SettingParam *param)
         else if (GTK_IS_ADJUSTMENT(el->widget))
             gtk_adjustment_set_value(GTK_ADJUSTMENT(el->widget), (gdouble)param->value);
     } else { /* combo box */
-        if (el->value == param->value)
+        if (el->value == param->value) {
             gtk_combo_box_set_active(GTK_COMBO_BOX(el->widget), el->x);
+        }
     }
 }
 
@@ -375,6 +443,56 @@ gboolean apply_current_preset_to_gui(gpointer data)
     return FALSE;
 }
 
+/**
+ * Free the data associated with the dynamically allocated settings
+ * for the EXP_POSITION.
+ */
+void modifier_settings_exp_free(EffectSettings *settings)
+{
+    guint i;
+    guint id = settings->id;
+    guint pos = settings->position;
+    gpointer key;
+    GList *link, *next, *orig_list;
+    WidgetTreeElem *el;
+    GObject *widget;
+
+    for (i = 0; i < 2; i++) {
+
+        id = settings[i].id;
+        pos = settings[i].position;
+        key = GINT_TO_POINTER(pos <<16| id);
+        orig_list = g_tree_lookup(widget_tree, key);
+        if (!orig_list) {
+            continue;
+        }
+
+        link = g_list_first(orig_list);
+        while (link != NULL) {
+            next = link->next;
+
+            el = link->data;
+            if (el) {
+
+                widget = el->widget;
+                if (g_object_get_data(G_OBJECT(widget), "exp")) {
+                    g_slice_free(WidgetTreeElem, el);
+                    link->data = NULL;
+                }
+            }
+            link = next;
+        }
+        
+        /* Remove all the list elements from which we freed data. */
+        orig_list = g_list_remove_all(orig_list, NULL);
+
+        /* The list should be empty at this point. */
+        g_assert(!orig_list);
+
+        /* Removed them all, so remove the key from the tree.*/
+        g_tree_steal(widget_tree, key);
+    }
+}
 /**
  *  \param settings effect parameters
  *  \param amt amount of effect parameters
@@ -424,6 +542,12 @@ GtkWidget *create_grid(EffectSettings *settings, gint amt, GHashTable *widget_ta
 
         widget_tree_add(G_OBJECT(adj), settings[x].id,
                         settings[x].position, -1, -1);
+
+        if (settings[x].position == EXP_POSITION) {
+            /* Tag the adj so we can free it when we free the modifier group. */
+            g_object_set_data(G_OBJECT(adj), "exp", GINT_TO_POINTER(1));
+        }
+
         gtk_grid_attach(GTK_GRID(grid), label, 0, x, 1, 1);
         gtk_grid_attach(GTK_GRID(grid), knob, 1, x, 1, 1);
         gtk_grid_attach(GTK_GRID(grid), widget, 2, x, 1, 1);
@@ -474,8 +598,9 @@ void effect_settings_group_free(EffectSettingsGroup *group)
 {
     if (group->child != NULL) {
         /* destroy widget without parent */
-        if (gtk_widget_get_parent(group->child) == NULL)
+        if (gtk_widget_get_parent(group->child) == NULL) {
             gtk_widget_destroy(group->child);
+        }
 
         g_object_unref(group->child);
     }
@@ -496,32 +621,40 @@ void combo_box_changed_cb(GtkComboBox *widget, gpointer data)
     EffectSettingsGroup *settings = NULL;
     gchar *name = NULL;
     gint x;
+
     g_object_get(G_OBJECT(widget), "active", &x, NULL);
 
     vbox = g_object_get_data(G_OBJECT(widget), "vbox");
 
     if (x != -1) {
+        GtkWidget *new_child = NULL;
+
         name = g_strdup_printf("SettingsGroup%d", x);
         settings = g_object_get_data(G_OBJECT(widget), name);
         g_free(name);
 
-        if (settings != NULL && allow_send)
-            set_option(settings->id, settings->position, settings->type);
-
         child = g_object_get_data(G_OBJECT(widget), "active_child");
-        if (child == settings->child) {
-            return;
+        if (settings != NULL)
+        {
+            if (allow_send)
+                set_option(settings->id, settings->position, settings->type);
+
+            if (child == settings->child) {
+                return;
+            }
         }
 
         if (child != NULL) {
             gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(gtk_widget_get_parent(vbox))), child);
         }
 
-        if (settings->child != NULL) {
+        if (settings != NULL && settings->child != NULL) {
             gtk_container_add(GTK_CONTAINER(gtk_widget_get_parent(gtk_widget_get_parent(vbox))), settings->child);
             gtk_widget_show_all(gtk_widget_get_parent(gtk_widget_get_parent(vbox)));
+            new_child = settings->child;
         }
-        g_object_set_data(G_OBJECT(widget), "active_child", settings->child);
+
+        g_object_set_data(G_OBJECT(widget), "active_child", new_child);
     }
 }
 
@@ -546,7 +679,7 @@ GtkWidget *create_widget_container(EffectGroup *group, gint amt, gint id, gint p
     gint x;
     gint cmbox_no = -1;
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
     widget_table = g_hash_table_new(g_direct_hash, g_direct_equal);
 
@@ -558,15 +691,22 @@ GtkWidget *create_widget_container(EffectGroup *group, gint amt, gint id, gint p
                 g_signal_connect(G_OBJECT(combo_box), "changed", G_CALLBACK(combo_box_changed_cb), group);
                 g_object_set_data(G_OBJECT(combo_box), "vbox", vbox);
             }
-            gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(combo_box),
-                                      NULL, group[x].label);
+
+            gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box), group[x].label);
             cmbox_no++;
 
             if ((group[x].settings != NULL) && (group[x].settings_amt > 0)) {
-                widget = create_grid(group[x].settings, group[x].settings_amt, widget_table);
+                /*
+                 * Create a grid for each combo box entry to contain per
+                 * combo box entry settings.
+                 */
+                widget = create_grid(group[x].settings,
+                                     group[x].settings_amt,
+                                     widget_table);
                 g_object_ref_sink(widget);
-            } else
+            } else {
                 widget = NULL;
+            }
 
             settings = g_slice_new(EffectSettingsGroup);
             settings->id = id;
@@ -577,11 +717,15 @@ GtkWidget *create_widget_container(EffectGroup *group, gint amt, gint id, gint p
             widget_tree_add(G_OBJECT(combo_box), id, position, group[x].type, x);
 
             name = g_strdup_printf("SettingsGroup%d", cmbox_no);
-            g_object_set_data_full(G_OBJECT(combo_box), name, settings, ((GDestroyNotify)effect_settings_group_free));
+            g_object_set_data_full(G_OBJECT(combo_box),
+                                   name, settings,
+                                   ((GDestroyNotify)effect_settings_group_free));
             g_free(name);
         } else {
             if ((group[x].settings != NULL) && (group[x].settings_amt > 0)) {
-                widget = create_grid(group[x].settings, group[x].settings_amt, widget_table);
+                widget = create_grid(group[x].settings,
+                                     group[x].settings_amt,
+                                     widget_table);
                 gtk_box_pack_end(GTK_BOX(vbox), widget, FALSE, TRUE, 0);
             }
         }
@@ -592,6 +736,157 @@ GtkWidget *create_widget_container(EffectGroup *group, gint amt, gint id, gint p
     return vbox;
 }
 
+/**
+ * Populate the vbox for an effect dependent on the modifier list.
+ *
+ * \param vbox          The vbox for the effect.
+ * \param combo_box     The combo box to repopulate.
+ * \param id            The id of the effect.
+ * \param position      The position of the effect.
+ */
+static void update_modifier_vbox(GtkWidget *vbox, GObject *combo_box, gint id, gint position)
+{
+    gint x;
+    EffectSettingsGroup *settings = NULL;
+    EffectGroup *group = get_modifier_group();
+    guint amt = get_modifier_amt();
+    GtkWidget *child = NULL;
+    GHashTable *widget_table;
+
+    widget_table = g_hash_table_new(g_direct_hash, g_direct_equal);
+
+    for (x = 0; x<amt; x++) {
+        gchar *name;
+        g_assert(group[x].label);
+
+        settings = g_slice_new(EffectSettingsGroup);
+        settings->id = id;
+        settings->type = group[x].type;
+        settings->position = position;
+
+        if (position == EXP_POSITION) {
+            child = g_object_steal_data(G_OBJECT(combo_box), "active_child");
+            child = create_grid(group[x].settings, group[x].settings_amt,
+                                widget_table);
+            g_object_ref_sink(child);
+            settings->child = child;
+        } else {
+            /* LFO has one settings group.*/
+            settings->child = NULL;
+        }
+
+        name = g_strdup_printf("SettingsGroup%d", x);
+        g_object_set_data(G_OBJECT(combo_box), name, settings);
+        g_free(name);
+
+        gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo_box), group[x].label);
+        widget_tree_add(combo_box, id, position, group[x].type, x);
+    }
+
+    g_hash_table_destroy(widget_table);
+
+    return;
+}
+
+static void clean_modifier_combo_box(GObject *combo_box, GList *list)
+{
+    EffectSettingsGroup *settings = NULL;
+    WidgetTreeElem *el;
+    gchar *name;
+    GList *link, *next, *stale_link;
+
+    link = g_list_first(list);
+
+    while (link != NULL) {
+        next = link->next;
+        el = link->data;
+        stale_link = NULL;
+
+        /* We need to clean the data associated with a combo box.
+         * This may include the per-entry settings widgets.
+         */
+        if (el->value != -1) {
+            /* This is a combo box entry. Remove the associated data. */
+            stale_link = link;
+            link = g_list_remove_link(list, link);
+            
+            name = g_strdup_printf("SettingsGroup%d", el->x);
+            settings = g_object_steal_data(G_OBJECT(combo_box), name);
+            if (settings && settings->child) {
+                gtk_widget_destroy(settings->child);
+            }
+
+            g_slice_free(EffectSettingsGroup, settings);
+            g_free(name);
+            g_slice_free(WidgetTreeElem, el);
+        }
+
+        if (stale_link) {
+            g_list_free_1(stale_link);
+        }
+
+        link = next;
+    }
+    gtk_combo_box_text_remove_all(GTK_COMBO_BOX_TEXT(combo_box));
+}
+
+/**
+ * Given a linkable effect, update the combo box for the linkable parameters.
+ *
+ * @param[in] pos Position
+ * @param[in] id Id
+ */
+void
+create_modifier_group (guint pos, guint id)
+{
+    
+    GtkWidget *vbox;
+    gpointer key;
+    WidgetTreeElem *el;
+    GList *list;
+    GObject *modifier_combo_box;
+
+    debug_msg(DEBUG_GROUP, "Building modifier group for position %d id %d \"%s\"",
+                           pos, id, get_xml_settings(id, pos)->label);
+
+    key = GINT_TO_POINTER((pos << 16) | id);
+    list = g_tree_lookup(widget_tree, key);
+
+    /* 
+     * The list will be destroyed and recreated, but we don't want to 
+     * handle the teardown ourselves. So steal it from the tree.
+     */
+    g_tree_steal(widget_tree, key);
+    if (!list) {
+        return;
+    }
+
+    el = g_list_nth_data(list, 0);
+    if (!el) {
+        g_warning("No effect settings group for position %d id %d!\n",
+                   pos, id);
+        return;
+    }
+
+    modifier_combo_box = el->widget;
+    g_assert(modifier_combo_box != NULL);
+
+    vbox = g_object_get_data(modifier_combo_box, "vbox");
+    g_assert(vbox != NULL);
+
+    clean_modifier_combo_box(modifier_combo_box, list);
+
+    update_modifier_vbox(vbox, modifier_combo_box, id, pos);
+
+    get_option(id, pos);
+
+    if (pos == EXP_POSITION) {
+        get_option(EXP_MIN, EXP_POSITION);
+        get_option(EXP_MAX, EXP_POSITION);
+    }
+}
+
+
 /**
  *  \param widgets Effect descriptions
  *  \param amt amount of effect descriptions
@@ -613,13 +908,14 @@ GtkWidget *create_vbox(Effect *widgets, gint amt, gchar *label)
 
     frame = gtk_frame_new(label);
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
     grid = gtk_grid_new();
     gtk_grid_set_column_spacing(GTK_GRID(grid), 2);
 
     for (x = 0; x<amt; x++) {
         if ((widgets[x].id != -1) && (widgets[x].position != -1)) {
+
             widget = create_on_off_button(&widgets[x]);
             gtk_grid_attach(GTK_GRID(grid), widget, 0, x, 1, 1);
 
@@ -629,21 +925,29 @@ GtkWidget *create_vbox(Effect *widgets, gint amt, gchar *label)
                 y = 0;
 
         } else if (widgets[x].label) {
+
             widget = gtk_label_new(widgets[x].label);
             gtk_grid_attach(GTK_GRID(grid), widget, 0, x, 1, 1);
             y = 0;
+
         } else {
+
             /* Default to 1 */
             if (x == 0)
                 y = 1;
         }
 
-        container = create_widget_container(widgets[x].group, widgets[x].group_amt, widgets[x].type, widgets[x].position);
+        container = create_widget_container(widgets[x].group,
+                                            widgets[x].group_amt,
+                                            widgets[x].type,
+                                            widgets[x].position);
+
         gtk_grid_attach(GTK_GRID(grid), container, 1-y, x+y, 1, 1);
     }
+    
     gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, FALSE, 2);
-
     gtk_container_add(GTK_CONTAINER(frame), vbox);
+
     return frame;
 }
 
@@ -844,23 +1148,31 @@ static void action_show_about_dialog_cb(GtkAction *action)
         "Stephen Rigler <riglersc@gmail.com>",
         "Jaco Kroon <jaco@kroon.co.za>",
         "Rafael Moreno <laocanfei@yahoo.com>",
-        "Andrew O. Shadoura <bugzilla@tut.by>",
+        "Andrew Shadura <bugzilla@tut.by>",
         "Andreas Karajannis <aakara13@googlemail.com>",
         "Miklos Aubert <miklos.aubert@gmail.com>",
         "Jonathan A. Tice <jonandtice@gmail.com>",
         "John Hammen <jhammen@gmail.com>",
         "Ahmed Toulan <thelinuxer@gmail.com>",
         "Tim LaBerge <tlaberge@visi.com>",
+        "Mauro Carvalho Chehab <maurochehab@gmail.com>",
         NULL
     };
-    static const gchar copyright[] = "Copyright \xc2\xa9 2009 Tomasz Moń";
+
+    static const gchar * const artists[] = {
+        "Islam Alwazery <wazery@ubuntu.com>",
+        NULL
+    };
+
+    static const gchar copyright[] = "Copyright \xc2\xa9 2009-2013 Tomasz Moń";
     static const gchar website[] = "http://desowin.org/gdigi/";
-    static const gchar version[] = "0.3.0";
+    static const gchar version[] = "0.4.0";
 
     GtkWidget *window = g_object_get_data(G_OBJECT(action), "window");
 
     gtk_show_about_dialog(GTK_WINDOW(window),
                           "authors", authors,
+                          "artists", artists,
                           "copyright", copyright,
                           "website", website,
                           "license-type", GTK_LICENSE_GPL_3_0,
@@ -891,7 +1203,7 @@ SupportedFileTypes file_types[] = {
 
 static guint n_file_types = G_N_ELEMENTS(file_types);
 
-gchar * 
+gchar *
 get_preset_filename (int prod_id)
 {
     return file_types[prod_id].name;
@@ -928,7 +1240,7 @@ static void action_open_preset_cb(GtkAction *action)
     for (x=0; x<n_file_types; x++) {
         GtkFileFilter *current_filter = gtk_file_filter_new();
         if (file_types[x].name == NULL) {
-            g_message("Skipping NULL array entry");
+            g_warning("Skipping NULL array entry");
             continue;
         }
 
@@ -938,7 +1250,6 @@ static void action_open_preset_cb(GtkAction *action)
 
         gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), current_filter);
         if (x == product_id) {
-            
             gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), current_filter);
         }
 
@@ -1008,10 +1319,9 @@ static void action_open_preset_cb(GtkAction *action)
                         section = get_genetx_section_id(genetx->version,
                                                         genetx->type);
                         bank = 0x04;
+                        index = genetx->channel;
 
-                        if (i == 0) {
-                            index = genetx->channel;
-                        } else {
+                        if (i != 0) {
                             if (genetx->channel == GENETX_CHANNEL1) {
                                 index = GENETX_CHANNEL1_CUSTOM;
                             } else if (genetx->channel == GENETX_CHANNEL2) {
@@ -1061,15 +1371,10 @@ static void action_save_preset_cb(GtkAction *action)
                                          GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                          NULL);
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-        GError *error = NULL;
         gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 
-
-
-        if (error) {
-            show_error_message(window, error->message);
-            g_error_free(error);
-            error = NULL;
+        if (filename == NULL) {
+            show_error_message(window, "No file name");
         } else {
             gchar real_filename[256];
             GList *list = get_current_preset();
@@ -1082,8 +1387,8 @@ static void action_save_preset_cb(GtkAction *action)
             write_preset_to_xml(preset, real_filename);
 
             preset_free(preset);
+            g_free(filename);
         }
-        g_free(filename);
     }
 
     gtk_widget_destroy(dialog);
@@ -1190,7 +1495,7 @@ static void add_menubar(GtkWidget *window, GtkWidget *vbox)
     gtk_window_add_accel_group(GTK_WINDOW(window), gtk_ui_manager_get_accel_group(ui));
 
     if (!gtk_ui_manager_add_ui_from_string(ui, menu_info, -1, &error)) {
-        g_message("building menus failed: %s", error->message);
+        g_warning("building menus failed: %s", error->message);
         g_error_free(error);
         error = NULL;
     }
@@ -1233,18 +1538,23 @@ void gui_create(Device *device)
     GtkWidget *widget;
     GtkWidget *notebook;
     GtkWidget *sw;             /* scrolled window to carry preset treeview */
+    GdkPixbuf *icon;
+
     gint x;
     gint i;
 
     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(window), "gdigi");
 
-    vbox = gtk_vbox_new(FALSE, 0);
+    icon = gdk_pixbuf_new_from_inline(-1, gdigi_icon, FALSE, NULL);
+    gtk_window_set_icon(GTK_WINDOW(window), icon);
+
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
     gtk_container_add(GTK_CONTAINER(window), vbox);
 
     add_menubar(window, vbox);
 
-    hbox = gtk_hbox_new(FALSE, 0);
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
     sw = gtk_scrolled_window_new(NULL, NULL);
@@ -1268,14 +1578,14 @@ void gui_create(Device *device)
 
     for (i = 0; i<device->n_pages; i++) {
         GtkWidget *label = NULL;
-        vbox = gtk_vbox_new(FALSE, 0);
+        vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
         label = gtk_label_new(device->pages[i].name);
 
         gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);
 
         for (x = 0; x<device->pages[i].n_effects; x++) {
             if ((x % ((device->pages[i].n_effects+1)/device->pages[i].n_rows)) == 0) {
-                hbox = gtk_hbox_new(FALSE, 0);
+                hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
                 gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 2);
             }
             widget = create_vbox(device->pages[i].effects[x].effect, device->pages[i].effects[x].amt, device->pages[i].effects[x].label);
@@ -1287,6 +1597,10 @@ void gui_create(Device *device)
     gtk_widget_show_all(window);
 
     g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_main_quit), NULL);
+
+    /* Get the initial values for the linkable parameters and the globals. */
+    send_message(REQUEST_MODIFIER_LINKABLE_LIST, "\x00\x01", 2);
+    send_message(REQUEST_GLOBAL_PARAMETERS, "\x00\x01", 2);
 }
 
 /**
@@ -1306,7 +1620,7 @@ void gui_free()
  *
  *  Displays dialogbox stating that device is unsupported.
  *
- *  \return TRUE if user selects "compability mode", otherwise FALSE.
+ *  \return TRUE if user selects "compatibility mode", otherwise FALSE.
  **/
 gboolean unsupported_device_dialog(Device **device)
 {
@@ -1329,7 +1643,7 @@ gboolean unsupported_device_dialog(Device **device)
 
     label = gtk_label_new("Your device appears to be unsupported by gdigi.\n"
                           "As some of the settings may be common between different devices,\n"
-                          "you can now select compability mode with one of the supported devices.\n"
+                          "you can now select compatibility mode with one of the supported devices.\n"
                           "Please take a look at gdigi's HACKING file.");
     gtk_container_add(GTK_CONTAINER(vbox), label);
 
@@ -1345,7 +1659,8 @@ gboolean unsupported_device_dialog(Device **device)
     if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
         gint number = gtk_combo_box_get_active(GTK_COMBO_BOX(combo_box));
         if (number != -1 && number <n_supported_devices) {
-            g_message("Starting %s compability mode", supported_devices[number]->name);
+            g_message("Starting %s compatibility mode",
+                      supported_devices[number]->name);
             *device = supported_devices[number];
             gtk_widget_destroy(dialog);
             return TRUE;
@@ -1365,7 +1680,6 @@ gboolean unsupported_device_dialog(Device **device)
  **/
 gint select_device_dialog (GList *devices)
 {
-
     GtkWidget *dialog;
     GtkWidget *label;
     GtkWidget *combo_box;
@@ -1385,13 +1699,14 @@ gint select_device_dialog (GList *devices)
 
     combo_box = gtk_combo_box_text_new();
     device = g_list_first(devices);
-    do {
+    while (device != NULL) {
         char *name;
 
         snd_card_get_longname(GPOINTER_TO_INT(device->data), &name);
         gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(combo_box), NULL, name);
 
-    } while ((device = g_list_next(device)));
+        device = g_list_next(device);
+    };
 
     gtk_container_add(GTK_CONTAINER(vbox), combo_box);
 
